@@ -1,8 +1,8 @@
 ''' 2D geometry primitives and implementation of some geometric algorithms. '''
 
 from typing import List, Sequence, Tuple
+import enum
 import collections.abc
-import dataclasses
 from dataclasses import dataclass
 import math
 from itertools import islice, cycle, groupby
@@ -136,7 +136,8 @@ class Point(metaclass=PointMeta):
 
         Return: The new Point instance
 
-        Raises: ValueError if the conversion was not successful.
+        Raises:
+            ValueError: If the conversion was not successful.
         '''
         if isinstance(value, Point):
             return value
@@ -161,6 +162,22 @@ class Line:
         pt1 (float): The first point of the line segment
         pt2 (float): The second point of the line segment
     '''
+
+    class Intersection(enum.Enum):
+        ''' The intersection type of two line segments. '''
+
+        LEFT = -1
+        ''' The second segment intersects the first one in left direction. '''
+
+        NONE = 0
+        ''' The two segments do not intersect. '''
+
+        RIGHT = 1
+        ''' The second segment intersects the first one in right direction. '''
+
+        def __bool__(self) -> bool:
+            return bool(self.value)
+
     pt1: Point
     ''' The first point of the line segment '''
 
@@ -171,20 +188,32 @@ class Line:
         if not isinstance(self.pt1, Point) or not isinstance(self.pt2, Point):
             raise ValueError('Line arguments "pt1" and "pt2" must be Point objects.')
 
-    def intersects(self, other: 'Line') -> bool:
+    def intersects(self, other: 'Line') -> Intersection:
         ''' Determines if this line segment intersects an other one.
 
+        The direction of intersection is interpreted as follows. Place an observer to the first
+        point of this line, looking to the second point of this line. If the second point of the
+        other line is on the left side, the directions of the intersection is "left", otherwise
+        it is "right". Attention: when considering the line intersection direction, keep in mind
+        that the geometry module uses the screen coordinate system orientation, i.e. the origin
+        can be found in the upper left corner of the screen.
+
         Args:
-            other: The other line segment
+            other (Line): The other line segment
 
         Returns:
-            `True` if the two segments intersect each other.
+            The line intersection type.
         '''
-        return (
-            Point.ccw(self.pt1, other.pt1, other.pt2) != Point.ccw(self.pt2, other.pt1, other.pt2)
-            and
-            Point.ccw(self.pt1, self.pt2, other.pt1) != Point.ccw(self.pt1, self.pt2, other.pt2)
-        )
+        if (
+            Point.ccw(self.pt1, other.pt1, other.pt2) == Point.ccw(self.pt2, other.pt1, other.pt2) or
+            Point.ccw(self.pt1, self.pt2, other.pt1) == Point.ccw(self.pt1, self.pt2, other.pt2)
+        ):
+            return Line.Intersection.NONE
+        else:
+            return (
+                Line.Intersection.LEFT if Point.ccw(self.pt1, self.pt2, other.pt1)
+                else Line.Intersection.RIGHT
+            )
 
     @classmethod
     def from_value(cls, value):
@@ -199,9 +228,11 @@ class Line:
         Args:
             value: the value to be converted
 
-        Returns: the Line instance
+        Returns:
+            The Line instance
 
-        Raises: ValueError if the value could not be converted to a Line.
+        Raises:
+            ValueError: If the value could not be converted to a Line.
         '''
         if isinstance(value, Line):
             return value
@@ -346,9 +377,11 @@ class Rectangle:
         Args:
             value: the tuple
 
-        Returns: the Rectangle instance
+        Returns:
+            The Rectangle instance
 
-        Raises: ValueError if the tuple could not be converted to a Rectangle.
+        Raises:
+            ValueError: If the tuple could not be converted to a Rectangle.
         '''
         if isinstance(value, Rectangle):
             return value
@@ -367,9 +400,11 @@ class Rectangle:
         Args:
             tlbr: A top-left-bottom-right sequence of floats.
 
-        Returns: the Rectangle instance
+        Returns:
+            The Rectangle instance
 
-        Raises: ValueError if the sequence could not be converted to a Rectangle.
+        Raises:
+            ValueError: If the sequence could not be converted to a Rectangle.
         '''
         if _issequence_or_numpy(tlbr) and len(tlbr) == 4:
             return cls(pt1=Point(x=tlbr[1], y=tlbr[0]), pt2=Point(x=tlbr[3], y=tlbr[2]))
@@ -383,9 +418,11 @@ class Rectangle:
         Args:
             tlbr: A top-left-width-height sequence of floats.
 
-        Returns: the Rectangle instance
+        Returns:
+            The Rectangle instance
 
-        Raises: ValueError if the sequence could not be converted to a Rectangle.
+        Raises:
+            ValueError: If the sequence could not be converted to a Rectangle.
         '''
         if _issequence_or_numpy(tlhw) and len(tlhw) == 4:
             return cls(
@@ -475,7 +512,7 @@ class PolyLine:
         if len(self.points) < 4:
             return True
 
-        # Iterate over consequitive point triplets
+        # Iterate over consecutive point triplets
         it0 = self.points
         it1 = islice(cycle(self.points), 1, None)
         it2 = islice(cycle(self.points), 2, None)
@@ -517,9 +554,11 @@ class PolyLine:
             value: the tuple
             closed: flags if the newly created PolyLine should be closed or not.
 
-        Returns: the PolyLine instance
+        Returns:
+            The PolyLine instance
 
-        Raises: ValueError if the tuple could not be converted to a PolyLine.
+        Raises:
+            ValueError: If the tuple could not be converted to a PolyLine.
         '''
         if isinstance(value, PolyLine):
             return value
