@@ -9,6 +9,7 @@ from itertools import islice
 from typing import List, Deque, Optional, Iterator, Dict, Any, Callable, Tuple
 from abc import ABC, abstractmethod
 import concurrent.futures
+import functools
 
 from dateutil.tz import tzlocal
 
@@ -16,7 +17,7 @@ def local_now() -> datetime.datetime:
     ''' Returns the current time in local time zone.
 
     Returns:
-        A timezone aware datettime instance in the local time zone.
+        A timezone aware datetime instance in the local time zone.
     '''
     return datetime.datetime.now(tz=tzlocal())
 
@@ -27,12 +28,12 @@ def local_dt(dt: datetime.datetime) -> datetime.datetime:
         dt: The naive datetime instance.
 
     Returns:
-        A timezone aware datettime instance in the local time zone.
+        A timezone aware datetime instance in the local time zone.
     '''
     return dt.astimezone(tz=tzlocal())
 
 def panorama_timestamp_to_datetime(panorama_ts: Tuple[int, int]) -> datetime.datetime:
-    ''' Converts panoramasdk.media.time_stamp (seconds, microsececonds)
+    ''' Converts panoramasdk.media.time_stamp (seconds, microseconds)
     tuple to python datetime.
 
     Args:
@@ -87,25 +88,28 @@ class BaseTimer(ABC):
         ''' Resets the timer. '''
         self.intervals.clear()
 
+    def _repr_intervals(self, intervals):
+        iv_list = [f'{iv:.4f}' for iv in islice(intervals, self.MAX_REPR_INTERVALS)]
+        if len(intervals) > self.MAX_REPR_INTERVALS:
+            iv_list.append('...')
+        return f'[{", ".join(iv_list)}]'
+
     def _repr_props(self) -> Iterator[str]:
         if self.intervals:
-            iv_list = [f'{iv:.4f}' for iv in islice(self.intervals, self.MAX_REPR_INTERVALS)]
-            if len(self.intervals) > self.MAX_REPR_INTERVALS:
-                iv_list.append('...')
-            yield f'intervals=[{", ".join(iv_list)}]'
+            yield f'intervals={self._repr_intervals(self.intervals)}'
             yield f'min={self.min():.4f}'
             yield f'mean={self.mean():.4f}'
             yield f'max={self.max():.4f}'
 
     def __repr__(self) -> str:
-        elems = [self.__class__.__name__] + list(self._repr_props())
-        return '<' + ' '.join(elems) + '>'
+        elements = [self.__class__.__name__] + list(self._repr_props())
+        return '<' + ' '.join(elements) + '>'
 
 
 class Ticker(BaseTimer):
 
     ''' A performance profiler that measures the time interval between repeatedly
-    occuring events.
+    occurring events.
 
     Ticker can also calculate basic statistics of the time intervals.
 
@@ -145,7 +149,7 @@ class StopWatch(BaseTimer):
     There are two ways to use StopWatch: as a context manager, or with the `tick()`
     method. You can use the same StopWatch object in both ways at the same time.
 
-    When used as a context managaer, StopWatch can be used to measure the
+    When used as a context manager, StopWatch can be used to measure the
     performance of python code using an elegant API based on context manager.
     You can measure nested and serial execution.
 
@@ -217,7 +221,7 @@ class StopWatch(BaseTimer):
             return self.children[name]
         if max_intervals is None:
             max_intervals = self.intervals.maxlen
-        child = StopWatch(name, max_intervals=max_intervals)
+        child = self.__class__(name, max_intervals=max_intervals)
         child.parent = self
         self.children[name] = child
         return child
@@ -248,6 +252,28 @@ class StopWatch(BaseTimer):
         self.intervals.append(time.perf_counter() - self._start)
         self._start = None
 
+    def measure(self, fun):
+        ''' Use stopwatch as decorator.
+
+        Usage::
+
+            import time
+            from backpack.timepiece import StopWatch
+
+            stopwatch = StopWatch('stopwatch')
+
+            @stopwatch.measure
+            def long_running_func():
+                time.sleep(10)
+
+        '''
+        @functools.wraps(fun)
+        def wrapper(*args, **kwds):
+            self.__enter__()
+            fun(*args, **kwds)
+            self.__exit__()
+        return wrapper
+
     @property
     def level(self) -> int:
         ''' Returns the number of parents. '''
@@ -255,8 +281,7 @@ class StopWatch(BaseTimer):
 
     def _repr_props(self) -> Iterator[str]:
         yield f'name={self.name}'
-        for prop in super()._repr_props():
-            yield prop
+        yield from super()._repr_props()
 
     def __repr__(self) -> str:
         ''' Indented string representation of this StopWatch.'''
@@ -275,11 +300,12 @@ class Callback:
 
     The callback can be optionally called asynchronously.
 
-    :param cb: The callback function to be called
-    :param cbargs: Positional arguments of the callback
-    :param cbkwargs: Keyword arguments of the callback
-    :param executor: If specified, the callback function will be called asynchronously, using
-        this executor.
+    Args:
+        cb (Callable): The callback function to be called
+        cbargs (Optional[List[Any]]): The callback function to be called
+        cbkwargs (Optional[Dict[str, Any]]): Keyword arguments of the callback
+        executor (Optional[concurrent.futures.Executor]): If specified, the callback function will
+            be called asynchronously using this executor.
     '''
 
     # pylint: disable=invalid-name,too-few-public-methods
@@ -311,8 +337,9 @@ class Schedule(ABC):
     The external scheduler is expected to call the `tick()` method periodically,
     most likely from an event-loop.
 
-    :param repeating: If this schedule fires repeatedly
-    :param callback: The callback to be called when the scheduler fires
+    Args:
+        repeating (bool): If this schedule fires repeatedly
+        callback (Callback): The callback to be called when the scheduler fires
     '''
 
     def __init__(
@@ -326,16 +353,18 @@ class Schedule(ABC):
     def fire(self) -> None:
         ''' Fires the schedule calling the callback.
 
-        :returns: If not using an executor (the callback is called synchronously), `fire()`
+        Returns:
+            If not using an executor (the callback is called synchronously), `fire()`
             returns the return value of the callback. Otherwise it returns None.
         '''
         return self.callback()
 
     @abstractmethod
-    def tick(self) -> bool: # pylint: disable=no-self-use
+    def tick(self) -> Tuple[bool, Any]: # pylint: disable=no-self-use
         ''' The heartbeat of the schedule to be called periodically.
 
-        :returns: A tuple of (True, callback_return_value) if the schedule was fired,
+        Returns:
+            A tuple of (True, callback_return_value) if the schedule was fired,
             otherwise (False, None)
         '''
 
@@ -345,16 +374,16 @@ class AtSchedule(Schedule):
 
     The task will be executed at the next tick after the specified datetime.
 
-    :param at: When to execute the task
-    :param args: Positional arguments to be passed to superclass initializer
-    :param kwargs: Keyword arguments to be passed to superclass initializer
+    Args:
+        at (datetime.datetime): When to execute the task
+        callback (Callback): The callback to be called when the scheduler fires
     '''
 
     # pylint: disable=invalid-name
     # Disabled for the `at` parameter
 
-    def __init__(self, at: datetime.datetime, *args: Any, **kwargs: Any):
-        super().__init__(False, *args, **kwargs)
+    def __init__(self, at: datetime.datetime, callback: Callback):
+        super().__init__(repeating=False, callback=callback)
         self.at_lock = threading.Lock()
         self.fire_lock = threading.Lock()
         self.at = at
@@ -379,7 +408,7 @@ class AtSchedule(Schedule):
             return (True, res)
         return (False, None)
 
-    def tick(self) -> bool:
+    def tick(self) -> Tuple[bool, Any]:
         if self.callback.executor:
             with self.fire_lock:
                 return self._do_tick()
@@ -392,13 +421,13 @@ class IntervalSchedule(Schedule):
     The task will be executed at the first tick and at each tick after
     the specified time interval has passed.
 
-    :param interval: The time interval of the executions
-    :param args: Positional arguments to be passed to superclass initializer
-    :param kwargs: Keyword arguments to be passed to superclass initializer
+    Args:
+        interval (datetime.timedelta): The time interval of the executions
+        callback (Callback): The callback to be called when the scheduler fires
     '''
 
-    def __init__(self, interval: datetime.timedelta, *args: Any, **kwargs: Any):
-        super().__init__(True, *args, **kwargs)
+    def __init__(self, interval: datetime.timedelta, callback: Callback):
+        super().__init__(repeating=True, callback=callback)
         self.interval = interval
         self._next_fire = None
 
@@ -406,7 +435,7 @@ class IntervalSchedule(Schedule):
         while self._next_fire <= now:
             self._next_fire += self.interval
 
-    def tick(self) -> bool:
+    def tick(self) -> Tuple[bool, Any]:
         now = local_now()
         if not self._next_fire:
             self._next_fire = now
@@ -423,20 +452,20 @@ class OrdinalSchedule(Schedule):
 
     At the first tick the task will not be executed.
 
-    :param ordinal: Execute the task once in every ordinal number of ticks. An OrdinalSchedule
-        with zero ordinal will never fire.
-    :param args: Positional arguments to be passed to superclass initializer
-    :param kwargs: Keyword arguments to be passed to superclass initializer
+    Args:
+        ordinal (int): Execute the task once in every ordinal number of ticks. An OrdinalSchedule
+            with zero ordinal will never fire.
+        callback (Callback): The callback to be called when the scheduler fires
     '''
 
-    def __init__(self, ordinal: int, *args: Any, **kwargs: Any):
-        super().__init__(True, *args, **kwargs)
+    def __init__(self, ordinal: int, callback: Callback):
+        super().__init__(repeating=True, callback=callback)
         if ordinal < 0:
             raise ValueError('ordinal must be greater or equal than zero')
         self.ordinal = ordinal
         self._counter = 0
 
-    def tick(self):
+    def tick(self) -> Tuple[bool, Any]:
         if self.ordinal == 0:
             return (False, None)
         self._counter += 1
@@ -451,7 +480,8 @@ class AlarmClock:
     ''' An alarm clock can be used to bundle different schedules and send them
     the tick event at once.
 
-    :param schedules: The list of the schedules.
+    Args:
+        schedules (List[Schedule]): The list of the schedules.
     '''
 
     # pylint: disable=too-few-public-methods
@@ -475,35 +505,29 @@ class AlarmClock:
             self.schedules.remove(schedule)
 
 
-class Tachometer:
-    ''' A Tachometer can be used to measure the frequency of recurring events,
-    and periodically report statistics about it.
+class BaseTachometer(ABC):
+    ''' Abstract base class for tachometers.
 
-    Call the `tick()` method of the tachometer each time an atomic event happens.
-    For example, if you are interested in the stastics of the frame processing
-    time of your application, call `tick` method each time you process a new
-    frame.
+    A Tachometer can be used to measure the frequency of recurring events, and periodically report
+    statistics about it by calling a callback function with the following signature::
 
-    Tachometer will periodically call a callback function with the following
-    signature::
-
-        def stats_callback(timestamp, ticker):
+        def stats_callback(timestamp: datetime.datetime, timer: BaseTimer):
             pass
 
-    passing the timestamp of the last event, as well as the `Ticker` instance that
+    passing the timestamp of the last event, as well as the `BaseTimer` instance that
     collected the events. You can access the `min()`, `max()`, `sum()` (total processing time)
-    and the `len()` (number of measurements) methods of the ticker.
+    and the `len()` (number of measurements) methods of the timer.
 
-    The `tick()` method will return a tuple with these elements:
-     - bool: True, if the internal schedule was fired (you can ignore this value)
-     - a tuple with these elements:
-       - bool: True, if the stats_callback was called
-       - the return value of stats_callback if it was called, else None
+    This class is not intended to be instantiated. Use one of the subclasses instead.
 
-    :param stats_callback: A callable with the above signature that will be called
-        when new statistics is available.
-    :param stat_interval: The interval of the statistics calculation
-    :param executor: If specified, callback will be called asynchronously using this executor
+    Args:
+        timer (BaseTimer): Instance of the BaseTimer subclass that will be used to report events.
+        stats_callback (Callable[[datetime.datetime, BaseTimer]):  A callable with the above
+            signature that will be called when new statistics is available.
+        stats_interval (datetime.timedelta): The interval of the statistics calculation. Defaults
+            to one minute.
+        executor (Optional[concurrent.futures.Executor]): If specified, callback will be called
+            asynchronously using this executor
     '''
 
     # pylint: disable=too-few-public-methods
@@ -513,7 +537,8 @@ class Tachometer:
 
     def __init__(
         self,
-        stats_callback: Callable[[datetime.datetime, 'Ticker'], None],
+        timer: BaseTimer,
+        stats_callback: Callable[[datetime.datetime, BaseTimer], None],
         stats_interval: datetime.timedelta = datetime.timedelta(seconds=60),
         executor: Optional[concurrent.futures.Executor] = None
     ):
@@ -522,21 +547,118 @@ class Tachometer:
             interval=stats_interval,
             callback=Callback(cb=self._calculate_stats, executor=executor)
         )
+        self.timer = timer
+
+    def _calculate_stats(self):
+        timestamp = local_now()
+        if len(self.timer.intervals) == 0:
+            return (False, None)
+        res = self.stats_callback(timestamp, self.timer)
+        self.timer.intervals.clear()
+        return (True, res)
+
+
+class TickerTachometer(BaseTachometer):
+    ''' TickerTachometer is a combination of a Ticker and a Tachometer.
+
+    It reports statistics about the call frequency of the `tick()` method.
+
+    Call the `tick()` method of the tachometer each time an atomic event happens.
+    For example, if you are interested in the spastics of the frame processing
+    time of your application, call `tick` method each time you process a new
+    frame.
+
+    Args:
+        stats_callback (Callable[[datetime.datetime, Ticker]):  A callable that will be called
+            when new statistics is available.
+        stats_interval (datetime.timedelta): The interval of the statistics calculation. Defaults
+            to one minute.
+        executor (Optional[concurrent.futures.Executor]): If specified, callback will be called
+            asynchronously using this executor
+    '''
+
+    def __init__(self,
+        stats_callback: Callable[[datetime.datetime, Ticker], None],
+        stats_interval: datetime.timedelta = datetime.timedelta(seconds=60),
+        executor: Optional[concurrent.futures.Executor] = None
+    ):
         ticker_intervals = int(self.EXPECTED_MAX_FPS * stats_interval.total_seconds())
         self.ticker = Ticker(max_intervals=ticker_intervals)
+        super().__init__(self.ticker, stats_callback, stats_interval, executor)
 
-    def tick(self) -> bool:
+    def tick(self) -> Tuple[bool, Any]:
         ''' Call this method when a recurring event happens.
 
-        :returns: True if the stat_callback was called in this tick.
+        Returns:
+            Tuple[bool, Any]: A tuple with these elements:
+
+                - bool: True, if stats_callback was called
+                - the return value of stats_callback if it was called, else None
         '''
         self.ticker.tick()
         return self.schedule.tick()
 
-    def _calculate_stats(self):
-        timestamp = local_now()
-        if len(self.ticker.intervals) == 0:
-            return (False, None)
-        res = self.stats_callback(timestamp, self.ticker)
-        self.ticker.intervals.clear()
-        return (True, res)
+
+class StopWatchTachometer(BaseTachometer):
+    ''' `StopWatchTachometer` is a combination of a StopWatch and a Tachometer.
+
+    It reports statistics about the elapsed time intervals.
+
+    Much like StopWatch, you can use `StopWatchTachometer` as a context manager or as a function
+    decorator.
+
+    For example::
+
+        swt = StopWatchTachometer(
+            stats_callback=lambda dt, timer: print('Mean interval:', timer.mean()),
+            stats_interval=datetime.timedelta(seconds=10)
+        )
+
+        for i in range(11):
+            with swt:
+                print('tick')
+                time.sleep(2)
+
+        @swt.measure
+        def long_running_func():
+            print('tack')
+            time.sleep(1.5)
+
+        for i in range(15):
+            long_running_func()
+
+    Args:
+        stats_callback (Callable[[datetime.datetime, Ticker]):  A callable that will be called
+            when new statistics is available.
+        stats_interval (datetime.timedelta): The interval of the statistics calculation. Defaults
+            to one minute.
+        executor (Optional[concurrent.futures.Executor]): If specified, callback will be called
+            asynchronously using this executor
+    '''
+
+    def __init__(self,
+        stats_callback: Callable[[datetime.datetime, Ticker], None],
+        stats_interval: datetime.timedelta = datetime.timedelta(seconds=60),
+        executor: Optional[concurrent.futures.Executor] = None
+    ):
+        ticker_intervals = int(self.EXPECTED_MAX_FPS * stats_interval.total_seconds())
+        self.stopwatch = StopWatch('tachometer', max_intervals=ticker_intervals)
+        super().__init__(self.stopwatch, stats_callback, stats_interval, executor)
+
+    def __enter__(self) -> 'StopWatchTachometer':
+        ''' Context manager entry point returning self.'''
+        self.stopwatch.__enter__()
+        return self
+
+    def __exit__(self, *_) -> None:
+        ''' Context manager exit. '''
+        self.stopwatch.__exit__()
+        self.schedule.tick()
+
+    def measure(self, fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwds):
+            self.__enter__()
+            fun(*args, **kwds)
+            self.__exit__()
+        return wrapper
